@@ -105,6 +105,7 @@ public:
   std::string output_result = "LightGBM_predict_result.txt";
   std::string convert_model = "gbdt_prediction.cpp";
   std::string input_model = "";
+  std::string model_format = "text";
   int verbosity = 1;
   int num_iteration_predict = -1;
   bool is_pre_partition = false;
@@ -163,7 +164,6 @@ struct ObjectiveConfig: public ConfigBase {
 public:
   virtual ~ObjectiveConfig() {}
   double sigmoid = 1.0f;
-  double huber_delta = 1.0f;
   double fair_c = 1.0f;
   // for Approximate Hessian With Gaussian
   double gaussian_eta = 1.0f;
@@ -178,6 +178,9 @@ public:
   int num_class = 1;
   // Balancing of positive and negative weights
   double scale_pos_weight = 1.0f;
+  // True will sqrt fit the sqrt(label)
+  bool reg_sqrt = false;
+  double alpha = 0.9f;
   LIGHTGBM_EXPORT void Set(const std::unordered_map<std::string, std::string>& params) override;
 };
 
@@ -187,8 +190,8 @@ public:
   virtual ~MetricConfig() {}
   int num_class = 1;
   double sigmoid = 1.0f;
-  double huber_delta = 1.0f;
   double fair_c = 1.0f;
+  double alpha = 0.9f;
   std::vector<double> label_gain;
   std::vector<int> eval_at;
   LIGHTGBM_EXPORT void Set(const std::unordered_map<std::string, std::string>& params) override;
@@ -360,8 +363,8 @@ struct ParameterAlias {
     {
       { "config", "config_file" },
       { "nthread", "num_threads" },
-      { "random_seed", "seed" },
       { "num_thread", "num_threads" },
+      { "random_seed", "seed" },
       { "boosting", "boosting_type" },
       { "boost", "boosting_type" },
       { "application", "objective" },
@@ -399,6 +402,7 @@ struct ParameterAlias {
       { "num_round", "num_iterations" },
       { "num_trees", "num_iterations" },
       { "num_rounds", "num_iterations" },
+      { "num_boost_round", "num_iterations" },
       { "sub_row", "bagging_fraction" },
       { "subsample", "bagging_fraction" },
       { "subsample_freq", "bagging_freq" },
@@ -426,9 +430,9 @@ struct ParameterAlias {
       { "cat_column", "categorical_column" },
       { "cat_feature", "categorical_column" },
       { "predict_raw_score", "is_predict_raw_score" },
-      { "predict_leaf_index", "is_predict_leaf_index" },
       { "raw_score", "is_predict_raw_score" },
       { "leaf_index", "is_predict_leaf_index" },
+      { "predict_leaf_index", "is_predict_leaf_index" },
       { "contrib", "is_predict_contrib" },
       { "predict_contrib", "is_predict_contrib" },
       { "min_split_gain", "min_gain_to_split" },
@@ -438,14 +442,14 @@ struct ParameterAlias {
       { "num_classes", "num_class" },
       { "unbalanced_sets", "is_unbalance" },
       { "bagging_fraction_seed", "bagging_seed" },
-      { "num_boost_round", "num_iterations" },
       { "workers", "machines" },
       { "nodes", "machines" },
+      { "subsample_for_bin", "bin_construct_sample_cnt" },
     });
     const std::unordered_set<std::string> parameter_set({
       "config", "config_file", "task", "device",
       "num_threads", "seed", "boosting_type", "objective", "data",
-      "output_model", "input_model", "output_result", "valid_data",
+      "output_model", "input_model", "output_result", "model_format", "valid_data",
       "is_enable_sparse", "is_pre_partition", "is_training_metric",
       "ndcg_eval_at", "min_data_in_leaf", "min_sum_hessian_in_leaf",
       "num_leaves", "feature_fraction", "num_iterations",
@@ -456,12 +460,12 @@ struct ParameterAlias {
       "ignore_column", "categorical_column", "is_predict_raw_score",
       "is_predict_leaf_index", "min_gain_to_split", "top_k",
       "lambda_l1", "lambda_l2", "num_class", "is_unbalance",
-      "max_depth", "subsample_for_bin", "max_bin", "bagging_seed",
+      "max_depth", "max_bin", "bagging_seed",
       "drop_rate", "skip_drop", "max_drop", "uniform_drop",
       "xgboost_dart_mode", "drop_seed", "top_rate", "other_rate",
       "min_data_in_bin", "data_random_seed", "bin_construct_sample_cnt",
       "num_iteration_predict", "pred_early_stop", "pred_early_stop_freq",
-      "pred_early_stop_margin", "use_missing", "sigmoid", "huber_delta",
+      "pred_early_stop_margin", "use_missing", "sigmoid",
       "fair_c", "poission_max_delta_step", "scale_pos_weight",
       "boost_from_average", "max_position", "label_gain",
       "metric", "metric_freq", "time_out",
@@ -472,7 +476,8 @@ struct ParameterAlias {
       "max_conflict_rate", "poisson_max_delta_step", "gaussian_eta",
       "histogram_pool_size", "output_freq", "is_provide_training_metric", "machine_list_filename", "machines",
       "zero_as_missing", "init_score_file", "valid_init_score_file", "is_predict_contrib",
-      "max_cat_threshold",  "cat_smooth", "min_data_per_group", "cat_l2", "max_cat_to_onehot"
+      "max_cat_threshold",  "cat_smooth", "min_data_per_group", "cat_l2", "max_cat_to_onehot",
+      "alpha", "reg_sqrt"
     });
     std::unordered_map<std::string, std::string> tmp_map;
     for (const auto& pair : *params) {
