@@ -3,10 +3,14 @@
 
 #include <LightGBM/meta.h>
 #include <LightGBM/dataset.h>
+#ifdef USE_PROTO
+#include "model.pb.h"
+#endif // USE_PROTO
 
 #include <string>
 #include <vector>
 #include <memory>
+#include <map>
 
 namespace LightGBM {
 
@@ -30,6 +34,13 @@ public:
   * \param str Model string
   */
   explicit Tree(const std::string& str);
+  #ifdef USE_PROTO
+  /*!
+  * \brief Construtor, from a protobuf object
+  * \param model_tree Model protobuf object
+  */
+  explicit Tree(const Model_Tree& model_tree);
+  #endif // USE_PROTO
 
   ~Tree();
 
@@ -108,8 +119,11 @@ public:
   * \return Prediction result
   */
   inline double Predict(const double* feature_values) const;
+  inline double PredictByMap(const std::unordered_map<int, double>& feature_values) const;
 
   inline int PredictLeafIndex(const double* feature_values) const;
+  inline int PredictLeafIndexByMap(const std::unordered_map<int, double>& feature_values) const;
+
 
   inline void PredictContrib(const double* feature_values, int num_features, double* output);
 
@@ -165,6 +179,11 @@ public:
   /*! \brief Serialize this object to if-else statement*/
   std::string ToIfElse(int index, bool is_predict_leaf_index) const;
 
+  #ifdef USE_PROTO
+  /*! \brief Serialize this object to protobuf object*/
+  void ToProto(Model_Tree& model_tree) const;
+  #endif // USE_PROTO
+
   inline static bool IsZero(double fval) {
     if (fval > -kZeroAsMissingValueRange && fval <= kZeroAsMissingValueRange) {
       return true;
@@ -292,6 +311,7 @@ private:
   * \return Leaf index
   */
   inline int GetLeaf(const double* feature_values) const;
+  inline int GetLeafByMap(const std::unordered_map<int, double>& feature_values) const;
 
   /*! \brief Serialize one node to json*/
   std::string NodeToJSON(int index) const;
@@ -299,6 +319,8 @@ private:
   /*! \brief Serialize one node to if-else statement*/
   std::string NodeToIfElse(int index, bool is_predict_leaf_index) const;
 
+  std::string NodeToIfElseByMap(int index, bool is_predict_leaf_index) const;
+
   double ExpectedValue() const;
 
   int MaxDepth();
@@ -425,6 +447,15 @@ inline double Tree::Predict(const double* feature_values) const {
   }
 }
 
+inline double Tree::PredictByMap(const std::unordered_map<int, double>& feature_values) const {
+  if (num_leaves_ > 1) {
+    int leaf = GetLeafByMap(feature_values);
+    return LeafOutput(leaf);
+  } else {
+    return leaf_value_[0];
+  }
+}
+
 inline int Tree::PredictLeafIndex(const double* feature_values) const {
   if (num_leaves_ > 1) {
     int leaf = GetLeaf(feature_values);
@@ -434,12 +465,21 @@ inline int Tree::PredictLeafIndex(const double* feature_values) const {
   }
 }
 
+inline int Tree::PredictLeafIndexByMap(const std::unordered_map<int, double>& feature_values) const {
+  if (num_leaves_ > 1) {
+    int leaf = GetLeafByMap(feature_values);
+    return leaf;
+  } else {
+    return 0;
+  }
+}
+
 inline void Tree::PredictContrib(const double* feature_values, int num_features, double* output) {
   output[num_features] += ExpectedValue();
   // Run the recursion with preallocated space for the unique path data
   if (num_leaves_ > 1) {
-    const int max_path_len = MaxDepth()+1;
-    PathElement *unique_path_data = new PathElement[(max_path_len*(max_path_len+1))/2];
+    const int max_path_len = MaxDepth() + 1;
+    PathElement *unique_path_data = new PathElement[(max_path_len*(max_path_len + 1)) / 2];
     TreeSHAP(feature_values, output, 0, 0, unique_path_data, 1, 1, -1);
     delete[] unique_path_data;
   }
@@ -450,8 +490,8 @@ inline void Tree::RecomputeLeafDepths(int node, int depth) {
   if (node < 0) {
     leaf_depth_[~node] = depth;
   } else {
-    RecomputeLeafDepths(left_child_[node], depth+1);
-    RecomputeLeafDepths(right_child_[node], depth+1);
+    RecomputeLeafDepths(left_child_[node], depth + 1);
+    RecomputeLeafDepths(right_child_[node], depth + 1);
   }
 }
 
@@ -469,6 +509,21 @@ inline int Tree::GetLeaf(const double* feature_values) const {
   return ~node;
 }
 
+inline int Tree::GetLeafByMap(const std::unordered_map<int, double>& feature_values) const {
+  int node = 0;
+  if (num_cat_ > 0) {
+    while (node >= 0) {
+      node = Decision(feature_values.count(split_feature_[node]) > 0 ? feature_values.at(split_feature_[node]) : 0.0f, node);
+    }
+  } else {
+    while (node >= 0) {
+      node = NumericalDecision(feature_values.count(split_feature_[node]) > 0 ? feature_values.at(split_feature_[node]) : 0.0f, node);
+    }
+  }
+  return ~node;
+}
+
+
 }  // namespace LightGBM
 
 #endif   // LightGBM_TREE_H_
