@@ -160,6 +160,21 @@ inline static const char* Atoi(const char* p, int* out) {
   return p;
 }
 
+template<class T>
+inline static double Pow(T base, int power) {
+  if (power < 0) {
+    return 1.0 / Pow(base, -power);
+  } else if (power == 0) {
+    return 1;
+  } else if (power % 2 == 0) {
+    return Pow(base*base, power / 2);
+  } else if (power % 3 == 0) {
+    return Pow(base*base*base, power / 3);
+  } else {
+    return base * Pow(base, power - 1);
+  }
+}
+
 inline static const char* Atof(const char* p, double* out) {
   int frac;
   double sign, value, scale;
@@ -168,7 +183,6 @@ inline static const char* Atof(const char* p, double* out) {
   while (*p == ' ') {
     ++p;
   }
-
   // Get sign, if any.
   sign = 1.0;
   if (*p == '-') {
@@ -187,13 +201,15 @@ inline static const char* Atof(const char* p, double* out) {
 
     // Get digits after decimal point, if any.
     if (*p == '.') {
-      double pow10 = 10.0;
+      double right = 0.0;
+      int nn = 0;
       ++p;
       while (*p >= '0' && *p <= '9') {
-        value += (*p - '0') / pow10;
-        pow10 *= 10.0;
+        right = (*p - '0') + right * 10.0;
+        ++nn;
         ++p;
       }
+      value += right / Pow(10.0, nn);
     }
 
     // Handle exponent, if any.
@@ -224,15 +240,16 @@ inline static const char* Atof(const char* p, double* out) {
   } else {
     size_t cnt = 0;
     while (*(p + cnt) != '\0' && *(p + cnt) != ' '
-      && *(p + cnt) != '\t' && *(p + cnt) != ','
-      && *(p + cnt) != '\n' && *(p + cnt) != '\r'
-      && *(p + cnt) != ':') {
+           && *(p + cnt) != '\t' && *(p + cnt) != ','
+           && *(p + cnt) != '\n' && *(p + cnt) != '\r'
+           && *(p + cnt) != ':') {
       ++cnt;
     }
     if (cnt > 0) {
       std::string tmp_str(p, cnt);
       std::transform(tmp_str.begin(), tmp_str.end(), tmp_str.begin(), Common::tolower);
-      if (tmp_str == std::string("na") || tmp_str == std::string("nan")) {
+      if (tmp_str == std::string("na") || tmp_str == std::string("nan") ||
+          tmp_str == std::string("null")) {
         *out = NAN;
       } else if (tmp_str == std::string("inf") || tmp_str == std::string("infinity")) {
         *out = sign * 1e308;
@@ -250,8 +267,6 @@ inline static const char* Atof(const char* p, double* out) {
   return p;
 }
 
-
-
 inline bool AtoiAndCheck(const char* p, int* out) {
   const char* after = Atoi(p, out);
   if (*after != '\0') {
@@ -582,11 +597,29 @@ static void ParallelSort(_RanIt _First, _RanIt _Last, _Pr _Pred) {
 // Check that all y[] are in interval [ymin, ymax] (end points included); throws error if not
 template <typename T>
 inline void CheckElementsIntervalClosed(const T *y, T ymin, T ymax, int ny, const char *callername) {
-  for (int i = 0; i < ny; ++i) {
-    if (y[i] < ymin || y[i] > ymax) {
-      std::ostringstream os;
-      os << "[%s]: does not tolerate element [#%i = " << y[i] << "] outside [" << ymin << ", " << ymax << "]";
-      Log::Fatal(os.str().c_str(), callername, i);
+  auto fatal_msg = [&y, &ymin, &ymax, &callername](int i) { 
+    std::ostringstream os;
+    os << "[%s]: does not tolerate element [#%i = " << y[i] << "] outside [" << ymin << ", " << ymax << "]";
+    Log::Fatal(os.str().c_str(), callername, i);
+  };
+  for (int i = 1; i < ny; i += 2) {
+    if (y[i - 1] < y[i]) {
+      if (y[i - 1] < ymin) {
+        fatal_msg(i - 1);
+      } else if (y[i] > ymax) {
+        fatal_msg(i);
+      }
+    } else {
+      if (y[i - 1] > ymax) {
+        fatal_msg(i - 1);
+      } else if (y[i] < ymin) {
+        fatal_msg(i);
+      }
+    }
+  }
+  if (ny & 1) { // odd
+    if (y[ny - 1] < ymin || y[ny - 1] > ymax) {
+      fatal_msg(ny - 1);
     }
   }
 }
@@ -595,17 +628,45 @@ inline void CheckElementsIntervalClosed(const T *y, T ymin, T ymax, int ny, cons
 // this is useful for checking weight requirements.
 template <typename T1, typename T2>
 inline void ObtainMinMaxSum(const T1 *w, int nw, T1 *mi, T1 *ma, T2 *su) {
-  T1 minw = w[0];
-  T1 maxw = w[0];
-  T2 sumw = static_cast<T2>(w[0]);
-  for (int i = 1; i < nw; ++i) {
-    sumw += w[i];
-    if (w[i] < minw) minw = w[i];
-    if (w[i] > maxw) maxw = w[i];
+  T1 minw;
+  T1 maxw;
+  T1 sumw;
+  int i;
+  if (nw & 1) { // odd
+    minw = w[0];
+    maxw = w[0];
+    sumw = w[0];
+    i = 2;
+  } else { // even
+    if (w[0] < w[1]) {
+      minw = w[0];
+      maxw = w[1];
+    } else {
+      minw = w[1];
+      maxw = w[0];
+    }
+    sumw = w[0] + w[1];
+    i = 3;
+  }
+  for (; i < nw; i += 2) {
+    if (w[i - 1] < w[i]) {
+      minw = std::min(minw, w[i - 1]);
+      maxw = std::max(maxw, w[i]);
+    } else {
+      minw = std::min(minw, w[i]);
+      maxw = std::max(maxw, w[i - 1]);
+    }
+    sumw += w[i - 1] + w[i];
+  }
+  if (mi != nullptr) {
+    *mi = minw;
+  }
+  if (ma != nullptr) {
+    *ma = maxw;
+  }
+  if (su != nullptr) {
+    *su = static_cast<T2>(sumw);
   }
-  if (mi != nullptr) *mi = minw;
-  if (ma != nullptr) *ma = maxw;
-  if (su != nullptr) *su = sumw;
 }
 
 template<class T>
@@ -632,6 +693,15 @@ inline bool FindInBitset(const uint32_t* bits, int n, T pos) {
   return (bits[i1] >> i2) & 1;
 }
 
+inline static bool CheckDoubleEqualOrdered(double a, double b) {
+  double upper = std::nextafter(a, INFINITY);
+  return b <= upper;
+}
+
+inline static double GetDoubleUpperBound(double a) {
+  return std::nextafter(a, INFINITY);;
+}
+
 }  // namespace Common
 
 }  // namespace LightGBM
