@@ -13,13 +13,14 @@
 #include <fstream>
 #include <memory>
 #include <mutex>
+#include <map>
 
 namespace LightGBM {
 
 /*!
 * \brief GBDT algorithm implementation. including Training, prediction, bagging.
 */
-class GBDT: public GBDTBase {
+class GBDT : public GBDTBase {
 public:
 
   /*!
@@ -186,11 +187,19 @@ public:
   void PredictRaw(const double* features, double* output,
                   const PredictionEarlyStopInstance* earlyStop) const override;
 
+  void PredictRawByMap(const std::unordered_map<int, double>& features, double* output,
+                       const PredictionEarlyStopInstance* early_stop) const override;
+
   void Predict(const double* features, double* output,
                const PredictionEarlyStopInstance* earlyStop) const override;
 
+  void PredictByMap(const std::unordered_map<int, double>& features, double* output,
+                    const PredictionEarlyStopInstance* early_stop) const override;
+
   void PredictLeafIndex(const double* features, double* output) const override;
 
+  void PredictLeafIndexByMap(const std::unordered_map<int, double>& features, double* output) const override;
+
   void PredictContrib(const double* features, double* output,
                       const PredictionEarlyStopInstance* earlyStop) const override;
 
@@ -236,6 +245,22 @@ public:
   */
   bool LoadModelFromString(const std::string& model_str) override;
 
+  #ifdef USE_PROTO
+  /*!
+  * \brief Save model with protobuf
+  * \param num_iterations Number of model that want to save, -1 means save all
+  * \param filename Filename that want to save to
+  */
+  void SaveModelToProto(int num_iteration, const char* filename) const override;
+
+  /*!
+  * \brief Restore from a serialized protobuf file
+  * \param filename Filename that want to restore from
+  * \return true if succeeded
+  */
+  bool LoadModelFromProto(const char* filename) override;
+  #endif // USE_PROTO
+
   /*!
   * \brief Calculate feature importances
   * \param num_iteration Number of model that want to use for feature importance, -1 means use all
